from ..vaultkeeper_adaptor import VKAdaptor


class TestAdaptor(object):

    def setup(self):
        self.DATABASES = {}
        self.BROKER_URL = ''
        self.adaptor = VKAdaptor(config_path='./test/testconfig.json',
                                 DATABASES=self.DATABASES,
                                 BROKER_URL=self.BROKER_URL)

    def test_get_broker_url(self):
        broker = self.adaptor.data[2]
        self.adaptor.get_broker_url(broker)
        assert (self.adaptor.BROKER_URL ==
                'amqp://rtest:rpass@0.0.0.0:5672/myvhost')

    def test_get_database_configs(self):
        db = self.adaptor.data[0]
        self.adaptor.get_database_configs(db)
        assert self.adaptor.DATABASES == {
             'default': {
                 'ENGINE': 'django.db.backends.postgresql_psycopg2',
                 'NAME': 'mydb',
                 'USER': 'ptest',
                 'PASSWORD': 'ppass',
                 'HOST': '0.0.0.0',
                 'PORT': 5432,
                 'SET_ROLE': 'app_owner',
                 'CONN_MAX_AGE': 0,
             }
        }

    def test_process_all(self):
        self.adaptor.process_all()
        assert self.adaptor.secrets == {
            'broker1': 'amqp://rtest:rpass@0.0.0.0:5672/myvhost',
            'default': {
                'ENGINE': 'django.db.backends.postgresql_psycopg2',
                'NAME': 'mydb',
                'USER': 'ptest',
                'PASSWORD': 'ppass',
                'HOST': '0.0.0.0',
                'PORT': 5432,
                'SET_ROLE': 'app_owner',
                'CONN_MAX_AGE': 0,
            },
            'auxiliary': {
                'ENGINE': 'django.db.backends.postgresql_psycopg2',
                'NAME': 'mydb1',
                'USER': 'ptest1',
                'PASSWORD': 'ppass1',
                'HOST': '1.1.1.1',
                'PORT': 5432,
                'SET_ROLE': 'app_owner1',
                'CONN_MAX_AGE': 0,
            }
        }



import dj_database_url
import json


def build_ids_url(backend, endpoint, identifier, secret):
    return '{0}{1}:{2}@{3}'.format(
        prefixes[backend.lower()],
        identifier,
        secret,
        endpoint
    )


class VKAdaptor(object):

    def __init__(self,
                 config_path=None,
                 DATABASES=None,
                 BROKER_URL=None,
                 ):
        self.config_path = config_path
        self.DATABASES = DATABASES
        self.BROKER_URL = BROKER_URL
        self.secrets = {}

        self.functionmap = {
            'postgresql': self.get_database_configs,
            'rabbitmq': self.get_broker_url,
        }

        try:
            with open(self.config_path) as f:
                self.data = json.load(f)
        except IOError as (errno, strerror):
            raise IOError("I/O error({0}): {1}".format(errno, strerror))

    def process_all(self):
        for entry in self.data:
            name = entry['id']
            backend = entry['backend']
            self.secrets[name] = self.functionmap[backend](entry)

    def get_database_configs(self, secret):
        url = build_ids_url(secret['backend'],
                            secret['endpoint'],
                            secret['username'],
                            secret['password'])
        d = dj_database_url.parse(url)
        if 'set_role' in secret.keys():
            d['SET_ROLE'] = secret['set_role']
        db = {secret['id']: d}
        self.DATABASES.update(db)
        return d

    def get_broker_url(self, secret):
        url = build_ids_url(secret['backend'],
                            secret['endpoint'],
                            secret['username'],
                            secret['password'])
        self.BROKER_URL = url
        return self.BROKER_URL


prefixes = {
    'postgresql': 'postgres://',
    'rabbitmq': 'amqp://'
}

db_backends = [
    'postgresql',
]



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages


with open('README.rst') as f:
    readme = f.read()

install_requires = [
                    'dj_database_url',
]

extras_require = {
        'test': [
            'pytest>=3.0.0',
            'responses',
            'pytest-cov',
        ]
}

setup(
    name='vaultkeeper_adaptor',
    version='0.0.1',
    description=('A small library that '
                 'allows Django apps to '
                 'consume Vaultkeeper output '
                 'as resource secrets.'),
    long_description=readme,
    install_requires=install_requires,
    extras_require=extras_require,
    author='mracter',
    author_email='mary@praekelt.org',
    url='https://github.com/praekeltfoundation/vaultkeeper',
    license='BSD',
    packages=find_packages(),
)
